@@ -11,6 +11,13 @@ test_that("date-times have informative types", {
   expect_equal(vec_ptype_full(new_duration(10)), "duration<secs>")
 })
 
+test_that("dates and times are vectors", {
+  expect_true(vec_is(Sys.Date()))
+  expect_true(vec_is(as.POSIXct("2020-01-01")))
+  expect_true(vec_is(as.POSIXlt("2020-01-01")))
+})
+
+
 # coerce ------------------------------------------------------------------
 
 test_that("datetime coercions are symmetric and unchanging", {
@@ -40,21 +47,21 @@ test_that("tz comes from first non-empty", {
   x <- as.POSIXct("2020-01-01")
   y <- as.POSIXct("2020-01-01", tz = "America/New_York")
 
-  expect_equal(vec_type2(x, y), y[0])
-  expect_equal(vec_type2(y, x), y[0])
+  expect_equal(vec_ptype2(x, y), y[0])
+  expect_equal(vec_ptype2(y, x), y[0])
 
   z <- as.POSIXct("2020-01-01", tz = "Pacific/Auckland")
-  expect_equal(vec_type2(y, z), y[0])
-  expect_equal(vec_type2(z, y), z[0])
+  expect_equal(vec_ptype2(y, z), y[0])
+  expect_equal(vec_ptype2(z, y), z[0])
 })
 
 test_that("POSIXlt always steered towards POSIXct", {
   dtc <- as.POSIXct("2020-01-01")
   dtl <- as.POSIXlt("2020-01-01")
 
-  expect_equal(vec_type2(dtc, dtl), dtc[0])
-  expect_equal(vec_type2(dtl, dtc), dtc[0])
-  expect_equal(vec_type2(dtl, dtl), dtc[0])
+  expect_equal(vec_ptype2(dtc, dtl), dtc[0])
+  expect_equal(vec_ptype2(dtl, dtc), dtc[0])
+  expect_equal(vec_ptype2(dtl, dtl), dtc[0])
 })
 
 
@@ -71,22 +78,23 @@ test_that("safe casts work as expected", {
   expect_equal(vec_cast(list(date), date), date)
 })
 
-test_that("lossy casts generate warning", {
+test_that("lossy casts generate error", {
   date <- as.Date("2018-01-01")
   datetime <- as.POSIXct(date) + c(0, 3600)
-
-  expect_condition(vec_cast(datetime, date), class = "warning_lossy_cast")
+  expect_lossy(vec_cast(datetime, date), vec_c(date, date), x = datetime, to = date)
 })
 
 test_that("invalid casts generate error", {
   date <- as.Date("2018-01-01")
-  expect_error(vec_cast(integer(), date), class = "error_incompatible_cast")
+  expect_error(vec_cast(integer(), date), class = "vctrs_error_incompatible_cast")
 })
 
-test_that("can cast NA", {
-  expect_equal(vec_cast(NA, new_date()), new_date(NA_real_))
+test_that("can cast NA and unspecified to Date", {
+  expect_identical(vec_cast(NA, new_date()), new_date(NA_real_))
+  expect_identical(vec_cast(unspecified(2), new_date()), new_date(dbl(NA, NA)))
 })
 
+
 # cast: datetimes -----------------------------------------------------------
 
 test_that("safe casts work as expected", {
@@ -108,11 +116,12 @@ test_that("safe casts work as expected", {
   expect_equal(vec_cast(datetime_l, datetime_l), datetime_l)
   expect_equal(vec_cast(as.Date(datetime_l), datetime_l), datetime_l)
   expect_equal(vec_cast(list(datetime_l), datetime_l), datetime_l)
+  expect_error(vec_cast(raw(), datetime_l), class = "vctrs_error_incompatible_cast")
 })
 
 test_that("invalid casts generate error", {
   datetime <- as.POSIXct("1970-02-01", tz = "UTC")
-  expect_error(vec_cast(integer(), datetime), class = "error_incompatible_cast")
+  expect_error(vec_cast(integer(), datetime), class = "vctrs_error_incompatible_cast")
 })
 
 test_that("dates become midnight in date-time tzone", {
@@ -129,11 +138,16 @@ test_that("dates become midnight in date-time tzone", {
   expect_equal(format(date2_l, "%H:%M"), "00:00")
 })
 
-test_that("can cast NA", {
-  expect_equal(vec_cast(NA, new_datetime()), new_datetime(NA_real_))
-  expect_equal(vec_cast(NA, as.POSIXlt(new_datetime())), new_datetime(NA_real_))
+test_that("can cast NA and unspecified to POSIXct and POSIXlt", {
+  dtc <- as.POSIXct("2020-01-01")
+  dtl <- as.POSIXlt("2020-01-01")
+  expect_identical(vec_cast(NA, dtc), vec_init(dtc))
+  expect_identical(vec_cast(NA, dtl), vec_init(dtl))
+  expect_identical(vec_cast(unspecified(2), dtc), vec_init(dtc, 2))
+  expect_identical(vec_cast(unspecified(2), dtl), vec_init(dtl, 2))
 })
 
+
 # cast: durations ------------------------------------------------------------
 
 test_that("safe casts work as expected", {
@@ -149,13 +163,15 @@ test_that("safe casts work as expected", {
 
 test_that("invalid casts generate error", {
   dt <- as.difftime(600, units = "secs")
-  expect_error(vec_cast(integer(), dt), class = "error_incompatible_cast")
+  expect_error(vec_cast(integer(), dt), class = "vctrs_error_incompatible_cast")
 })
 
-test_that("can cast NA", {
-  expect_equal(vec_cast(NA, new_duration()), new_duration(NA_real_))
+test_that("can cast NA and unspecified to duration", {
+  expect_identical(vec_cast(NA, new_duration()), new_duration(na_dbl))
+  expect_identical(vec_cast(unspecified(2), new_duration()), new_duration(dbl(NA, NA)))
 })
 
+
 # arithmetic --------------------------------------------------------------
 
 test_that("default is error", {
@@ -164,25 +180,28 @@ test_that("default is error", {
   t <- as.difftime(12, units = "hours")
   f <- factor("x")
 
-  expect_error(vec_arith("+", d, f), class = "error_incompatible_op")
-  expect_error(vec_arith("+", dt, f), class = "error_incompatible_op")
-  expect_error(vec_arith("+", t, f), class = "error_incompatible_op")
+  expect_error(vec_arith("+", d, f), class = "vctrs_error_incompatible_op")
+  expect_error(vec_arith("+", dt, f), class = "vctrs_error_incompatible_op")
+  expect_error(vec_arith("+", t, f), class = "vctrs_error_incompatible_op")
+  expect_error(vec_arith("*", dt, t), class = "vctrs_error_incompatible_op")
+  expect_error(vec_arith("*", d, t), class = "vctrs_error_incompatible_op")
+  expect_error(vec_arith("!", t, MISSING()), class = "vctrs_error_incompatible_op")
 })
 
 test_that("date-time vs date-time", {
   d <- as.Date("2018-01-01")
   dt <- as.POSIXct(d)
 
-  expect_error(vec_arith("+", d, d), class = "error_incompatible_op")
+  expect_error(vec_arith("+", d, d), class = "vctrs_error_incompatible_op")
   expect_equal(vec_arith("-", d, d), d - d)
 
-  expect_error(vec_arith("+", d, dt), class = "error_incompatible_op")
+  expect_error(vec_arith("+", d, dt), class = "vctrs_error_incompatible_op")
   expect_equal(vec_arith("-", d, dt), difftime(d, dt))
 
-  expect_error(vec_arith("+", dt, d), class = "error_incompatible_op")
+  expect_error(vec_arith("+", dt, d), class = "vctrs_error_incompatible_op")
   expect_equal(vec_arith("-", dt, d), difftime(dt, d))
 
-  expect_error(vec_arith("+", dt, dt), class = "error_incompatible_op")
+  expect_error(vec_arith("+", dt, dt), class = "vctrs_error_incompatible_op")
   expect_equal(vec_arith("-", dt, dt), dt - dt)
 })
 
@@ -193,26 +212,34 @@ test_that("date-time vs numeric", {
    expect_equal(vec_arith("+", d, 1), d + 1)
    expect_equal(vec_arith("+", 1, d), d + 1)
    expect_equal(vec_arith("-", d, 1), d - 1)
-   expect_error(vec_arith("-", 1, d), class = "error_incompatible_op")
+   expect_error(vec_arith("-", 1, d), class = "vctrs_error_incompatible_op")
 
-   expect_error(vec_arith("*", 1, d), class = "error_incompatible_op")
-   expect_error(vec_arith("*", d, 1), class = "error_incompatible_op")
+   expect_error(vec_arith("*", 1, d), class = "vctrs_error_incompatible_op")
+   expect_error(vec_arith("*", d, 1), class = "vctrs_error_incompatible_op")
 })
 
 test_that("date-time vs difftime", {
   d <- as.Date("2018-01-01")
   dt <- as.POSIXct(d)
   t <- as.difftime(1, units = "days")
+  th <- as.difftime(c(1, 24), units = "hours")
 
   expect_equal(vec_arith("+", dt, t), dt + t)
   expect_equal(vec_arith("+", d, t), d + t)
+  expect_equal(vec_arith("+", dt, th), dt + th)
+  expect_lossy(vec_arith("+", d, th), d + th, x = t, to = d)
   expect_equal(vec_arith("-", dt, t), dt - t)
   expect_equal(vec_arith("-", d, t), d - t)
+  expect_equal(vec_arith("-", dt, th), dt - th)
+  expect_lossy(vec_arith("-", d, th), d - th, x = t, to = d)
 
   expect_equal(vec_arith("+", t, dt), dt + t)
   expect_equal(vec_arith("+", t, d), d + t)
-  expect_error(vec_arith("-", t, dt), class = "error_incompatible_op")
-  expect_error(vec_arith("-", t, d), class = "error_incompatible_op")
+  expect_equal(vec_arith("+", th, dt), dt + th)
+  expect_lossy(vec_arith("+", th, d), d + th, x = t, to = d)
+
+  expect_error(vec_arith("-", t, dt), class = "vctrs_error_incompatible_op")
+  expect_error(vec_arith("-", t, d), class = "vctrs_error_incompatible_op")
 })
 
 test_that("difftime vs difftime/numeric", {
@@ -231,10 +258,10 @@ test_that("difftime vs difftime/numeric", {
 
   expect_equal(vec_arith("*", 2, t), 2 * t)
   expect_equal(vec_arith("*", t, 2), 2 * t)
-  expect_error(vec_arith("*", t, t), class = "error_incompatible_op")
+  expect_error(vec_arith("*", t, t), class = "vctrs_error_incompatible_op")
 
   expect_equal(vec_arith("/", t, 2), t / 2)
-  expect_error(vec_arith("/", 2, t), class = "error_incompatible_op")
+  expect_error(vec_arith("/", 2, t), class = "vctrs_error_incompatible_op")
 
   expect_equal(vec_arith("/", t, t), 1)
   expect_equal(vec_arith("%/%", t, t), 1)
@@ -245,6 +272,6 @@ test_that("difftime vs difftime/numeric", {
 # Math --------------------------------------------------------------------
 
 test_that("date and date times don't support math", {
-  expect_error(vec_math("sum", new_date()), class = "error_unsupported")
-  expect_error(vec_math("sum", new_datetime()), class = "error_unsupported")
+  expect_error(vec_math("sum", new_date()), class = "vctrs_error_unsupported")
+  expect_error(vec_math("sum", new_datetime()), class = "vctrs_error_unsupported")
 })
